#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "cmpfit-1.2/mpfit.h"
#include "var_struct.h"
#include "read_data.h"
#include "fitujeme.h"
#include "misc.h"
#include "deriv.h"
#include "filter_zeropoints.h"
#include "bolometric.h"

using namespace std;

const double Lsun = 3.9e33;
const double c =2.99792458e10;
const double pc=3.086e18;
const double PI = 3.14159265359;

extern double global_covar[1000][1000];


double mni(vars_struct data, double *a, const int star_no, const int phot_band, const double time) {
	// phot_band is dummy, used for compatibility
	// filters entering bolometric calculation in the order of increasing central wavelength
	const int nbol_flt = 12;
	int bol_flt[nbol_flt] = {13, 14, 15, 16, 0, 1, 2, 3, 4, 5, 6, 7};
	double *flt_zero = new double[data.n_phot_bands], *fluxes = new double[nbol_flt];
	// define filter zeropoints
	define_filter_zeropoints(flt_zero, data.filter_lambda_center, data.n_phot_bands);
	double theo_mag, theo_vel, wt_exp, rho, temp;
	for (int i=0;i<nbol_flt;i++) {
		calculate_mag_vel(data, a, star_no, time, bol_flt[i], 0, 1, theo_mag, theo_vel, wt_exp, rho, temp);
		fluxes[i] = flt_zero[bol_flt[i]]*pow(10.0, -0.4*(theo_mag-30.0))*1e4; //factor 1e4 is due to conversion from Angstrom to microns
	}
	int i_bolo_start=0;
	//if (temp < -0.06) {i_bolo_start = 4;}	// include Swift only if tau > -0.05 (where it is well-defined)
	double bolo = 0.0;
	for (int i=i_bolo_start;i<nbol_flt-1;i++) {
		bolo += 0.5*(fluxes[i+1]+fluxes[i])*(data.filter_lambda_center[bol_flt[i+1]]-data.filter_lambda_center[bol_flt[i]]);
	}
	bolo += fluxes[nbol_flt-1]*data.filter_lambda_center[bol_flt[nbol_flt-1]]/3.0;
	bolo *= 4.0*PI*(10.0*pc)*(10.0*pc);
	delete[] flt_zero;
	delete[] fluxes;
	return (bolo)*7.866e-44*exp((200.-6.1)/111.26);
}




void calculate_mni(vars_struct data, double *a, double &mni_out, double &mni_err, double &l50, double &l50_err) {
	int n_coef = data.n_per_filter*data.n_phot_bands+data.n_velo_lines+data.n_gal+data.n_per_star*data.n_stars;

	int n_global = data.n_per_filter*data.n_phot_bands+data.n_velo_lines;
	double **covar_global = new double*[n_global];
	for(int i = 0; i < n_global; ++i) {
		covar_global[i] = new double[n_global];
		for (int j=0;j<n_global;j++) covar_global[i][j] = 0.0;
	}

	read_covar_global(covar_global, n_global, "covar_global.dat");


	double time, mni_val, mni_sig, *pder=new double[n_coef], m50_val, m50_sig;

	int phot_band = 2;
	int star_no = 0;
	int star_start =data.n_per_filter*data.n_phot_bands+data.n_velo_lines+data.n_gal+data.n_per_star*star_no;

	time = a[star_start+0]+200.0;
	mni_val = mni(data, a, star_no, phot_band, time);
	for (int i=0;i<n_coef;i++) pder[i] = gsl_deriv_central(mni, a, n_coef, i, 1e-4, data, star_no, phot_band, time);

	mni_sig = 0.0;
	for (int i=star_start;i<n_coef;i++) for (int j=star_start;j<n_coef;j++) mni_sig += global_covar[i][j]*pder[i]*pder[j];
	for (int i=0;i<n_global;i++) for (int j=0;j<n_global;j++) mni_sig += covar_global[i][j]*pder[i]*pder[j];

	time = a[star_start+0]+50.0;
	m50_val = bolometric(data, a, star_no, phot_band, time);
	for (int i=0;i<n_coef;i++)	pder[i] = gsl_deriv_central(bolometric, a, n_coef, i, 1e-4, data, star_no, phot_band, time);

	m50_sig = 0.0;
	//for (int i=star_start;i<n_coef;i++) for (int j=star_start;j<n_coef;j++) m50_sig += global_covar[i][j]*pder[i]*pder[j];
	for (int i=0;i<n_global;i++) for (int j=0;j<n_global;j++) m50_sig += covar_global[i][j]*pder[i]*pder[j];

	mni_out = mni_val;
	mni_err = sqrt(mni_sig);
	l50 = m50_val;
	l50_err = sqrt(m50_sig);

	delete[] pder;
}
